searchData={"items":[{"type":"module","title":"JokenJwks","doc":"`Joken.Hooks` implementation for fetching `Joken.Signer`s from public JWKS URLs.\n\nThis hook is intended to be used when you are _verifying_ a token is signed with\na well known public key. It only overrides the `before_verify/2` callback providing a\n`Joken.Signer` for the given token. It is important to notice this is not meant for\nuse when **GENERATING** a token. So, using this hook with `Joken.encode_and_sign`\nfunction **WILL NOT WORK!!!**\n\nTo use it, pass this hook to Joken either with the `add_hook/2` macro or directly\nto each `Joken` function. Example:\n\n    defmodule MyToken do\n      use Joken.Config\n\n      add_hook(JokenJwks, strategy: MyFetchingStrategy)\n\n      # rest of your token config\n    end\n\nOr:\n\n    Joken.verify_and_validate(config, token, nil, context, [{JokenJwks, strategy: MyStrategy}])","ref":"JokenJwks.html"},{"type":"module","title":"Fetching strategy - JokenJwks","doc":"Very rarely, your authentication server might rotate or block its keys. Key rotation is the\nprocess of issuing a new key that in time will replace the older key. This is security hygiene\nand should/might be a regular process.\n\nSometimes it is important to block keys because they got leaked or for any other reason.\n\nOther times you simply don't control the authentication server and can't ensure the keys won't\nchange. This is the most common scenario for this hook.\n\nIn these cases (and some others) it is important to have a cache invalidation strategy: all your\ncached keys should be refreshed. Since the best strategy might differ for each use case, there\nis a behaviour that can be customized as the \"fetching strategy\", that is: when to fetch and re-fetch\nkeys. `JokenJwks` has a default strategy that tries to be smart and cover most use cases by default.\nIt combines a time based state machine to avoid overflowing the system with re-fetching keys. If  that\nis not a good option for your use case, it can still be configured. Please, see\n`JokenJwks.SignerMatchStrategy` or `JokenJwks.DefaultStrategyTemplate` docs for more information.","ref":"JokenJwks.html#module-fetching-strategy"},{"type":"module","title":"JokenJwks.DefaultStrategyTemplate","doc":"A `JokenJwks.SignerMatchStrategy` template that has a window of time for refreshing its\ncache. This is a template and not a concrete implementation. You should `use` this module\nin order to use the default strategy.\n\nThis implementation is a task that should be supervised. It loops on a time window checking\nwhether it should re-fetch keys or not.\n\nEvery time a bad kid is received it writes to an ets table a counter to 1. When the task\nloops, it polls for the counter value. If it is more than zero it starts re-fetching the\ncache. Upon successful fetching, it zeros the counter once again. This way we avoid\noverloading the JWKS server.\n\nIt will try to fetch signers when supervision starts it. This can be a sync or async operation\ndepending on the value of `first_fetch_sync`. It defaults to `false`.","ref":"JokenJwks.DefaultStrategyTemplate.html"},{"type":"module","title":"Resiliency - JokenJwks.DefaultStrategyTemplate","doc":"This strategy tries to be smart about keys it can USE to verify signatures. For example, if the\nprovider has encryption keys, it will skip those (any key with field \"use\" with value \"enc\").\n\nAlso, if the running BEAM instance has no support for a given signature algorithm (possibly not implemented\non the given OpenSSL + BEAM + JOSE combination) this implementation will also skip those.\n\nBe sure to check your logs as if there are NO signers available it will log a warning telling you\nthat.\n\nFor debugging purpouses, calling the function `fetch_signers/2` directly might be helpful.","ref":"JokenJwks.DefaultStrategyTemplate.html#module-resiliency"},{"type":"module","title":"Usage - JokenJwks.DefaultStrategyTemplate","doc":"This strategy must be under your apps' supervision tree. It must be explicitly used under a\nmodule so that you can have more than one JWKS source.\n\nWhen using this strategy, there is an `init_opts/1` callback that can be overridden. This is called\nupon supervision start. It should return a keyword list with all the options. This follows the\nstandard practice of allowing a callback for using runtime configuration. It can override all\nother options as this has higher preference.","ref":"JokenJwks.DefaultStrategyTemplate.html#module-usage"},{"type":"module","title":"Configuration - JokenJwks.DefaultStrategyTemplate","doc":"Other than the `init_opts/1` callback you can pass options through `Config` and when starting\nthe supervisor. The order of preference in least significant order is:\n\n  - Per environment `Config`\n  - Supervisor child options\n  - `init_opts/1` callback\n\nThe only mandatory option is `jwks_url` (`binary()`) that is, usually, a\nruntime parameter like a system environment variable. It is recommended to\nuse the `init_opts/1` callback.\n\nOther options are:\n\n  - `time_interval` (`integer()` - default 60_000 (1 minute)): time interval\n    for polling if it is needed to re-fetch the keys\n\n  - `should_start` (`boolean()` - default `true`): whether to start the\n    supervised polling task. For tests, this should be false\n\n  - `first_fetch_sync` (`boolean()` - default `false`): whether to fetch the\n    first time synchronously or async\n\n  - `explicit_alg` (`String.t()`): the JWS algorithm for use with the key.\n    Overrides the one in the JWK\n\n  - `http_max_retries_per_fetch` (`pos_integer()` - default `10`): passed to\n    `Tesla.Middleware.Retry`\n\n  - `http_delay_per_retry` (`pos_integer()` - default `500`): passed to\n    `Tesla.Middleware.Retry`","ref":"JokenJwks.DefaultStrategyTemplate.html#module-configuration"},{"type":"module","title":"Examples - JokenJwks.DefaultStrategyTemplate","doc":"defmodule JokenExample.MyStrategy do\n      use JokenJwks.DefaultStrategyTemplate\n\n      def init_opts(opts) do\n        url = # fetch url ...\n        Keyword.merge(opts, jwks_url: url)\n      end\n    end\n\n    defmodule JokenExample.Application do\n      @doc false\n      def start(_type, _args) do\n        import Supervisor.Spec, warn: false\n\n        children = [\n          {MyStrategy, time_interval: 2_000}\n        ]\n\n        opts = [strategy: :one_for_one]\n        Supervisor.start_link(children, opts)\n      end\n    end\n\nThen on your token configuration module:\n\n    defmodule MyToken do\n      use Joken.Config\n\n      add_hook(JokenJwks, strategy: MyStrategy)\n      # rest of your token config\n    end","ref":"JokenJwks.DefaultStrategyTemplate.html#module-examples"},{"type":"module","title":"Telemetry events - JokenJwks.DefaultStrategyTemplate","doc":"This library produces events for helping understand its behaviour. Event prefix is\n`[:joken_jwks, :default_strategy]`. It always add the module as a metadata.\n\nThis can be useful to implement logging.\n\nEvents:\n\n- `[:joken_jwks, :default_strategy, :refetch]`: starts refetching\n- `[:joken_jwks, :default_strategy, :signers]`: signers sucessfully fetched\n- `[:joken_jwks, :http_fetcher, :start | :stop | :exception]`: http lifecycle","ref":"JokenJwks.DefaultStrategyTemplate.html#module-telemetry-events"},{"type":"function","title":"JokenJwks.DefaultStrategyTemplate.fetch_signers/3","doc":"Fetch signers with `JokenJwks.HttpFetcher`","ref":"JokenJwks.DefaultStrategyTemplate.html#fetch_signers/3"},{"type":"module","title":"JokenJwks.DefaultStrategyTemplate.EtsCache","doc":"Simple ETS counter based state machine","ref":"JokenJwks.DefaultStrategyTemplate.EtsCache.html"},{"type":"function","title":"JokenJwks.DefaultStrategyTemplate.EtsCache.check_state/1","doc":"Returns 0 - no need to fetch signers or 1 - need to fetch","ref":"JokenJwks.DefaultStrategyTemplate.EtsCache.html#check_state/1"},{"type":"function","title":"JokenJwks.DefaultStrategyTemplate.EtsCache.get_signers/1","doc":"Loads fetched signers","ref":"JokenJwks.DefaultStrategyTemplate.EtsCache.html#get_signers/1"},{"type":"function","title":"JokenJwks.DefaultStrategyTemplate.EtsCache.new/1","doc":"Starts ETS cache - will only create if table doesn't exist already","ref":"JokenJwks.DefaultStrategyTemplate.EtsCache.html#new/1"},{"type":"function","title":"JokenJwks.DefaultStrategyTemplate.EtsCache.put_signers/2","doc":"Puts fetched signers","ref":"JokenJwks.DefaultStrategyTemplate.EtsCache.html#put_signers/2"},{"type":"function","title":"JokenJwks.DefaultStrategyTemplate.EtsCache.set_status/2","doc":"Sets the cache status","ref":"JokenJwks.DefaultStrategyTemplate.EtsCache.html#set_status/2"},{"type":"module","title":"JokenJwks.HttpFetcher","doc":"Makes a GET request to an OpenID Connect certificates endpoint.\n\nThis must be a standard JWKS URI as per the specification here:\nhttps://openid.net/specs/openid-connect-discovery-1_0.html#ProviderMetadata\n\nThis uses the `Tesla` library to make it easy to test or change the adapter\nif wanted.\n\nOptions include:\n\n- `:http_adapter` (default: `Tesla.Adapter.Hackney`)\n- `:http_middlewares`: a list of extra `Tesla.Middleware` configurations\n\nSee our tests for an example of mocking the HTTP fetching.","ref":"JokenJwks.HttpFetcher.html"},{"type":"function","title":"JokenJwks.HttpFetcher.fetch_signers/2","doc":"Fetches the JWKS signers from the given url.\n\nThis retries up to 10 times with a fixed delay of 500 ms until the server\ndelivers an answer. We only perform a GET request that is idempotent.\n\nWe use `:hackney` as it validates certificates automatically.","ref":"JokenJwks.HttpFetcher.html#fetch_signers/2"},{"type":"behaviour","title":"JokenJwks.SignerMatchStrategy","doc":"A strategy behaviour for using with `JokenJwks`.\n\nJokenJwks will call this for every token with a kid. It is the strategy's responsibility to handle\ncaching and matching of the kid with its signers cache.\n\nSee `JokenJwks.DefaultStrategyTemplate` for an implementation.","ref":"JokenJwks.SignerMatchStrategy.html"},{"type":"callback","title":"JokenJwks.SignerMatchStrategy.match_signer_for_kid/2","doc":"","ref":"JokenJwks.SignerMatchStrategy.html#c:match_signer_for_kid/2"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [Unreleased]\n\n## [1.7.0-rc.0]","ref":"changelog.html"},{"type":"extras","title":"BREAKING CHANGES - Changelog","doc":"- No more custom logging (use Logger facilities)\n\nIf you need to disable specific logging from the library, you can use default `Logger` facilities like `Logger.put_application_level/2` (which accepts `:none`) and so on. There is no need to use custom logging here anymore.\n\n- No more custom telemetry (use Teslas built-in)\n\nTesla has its own middleware for telemetry events. We should just use that :) \n\n- Different process strategy (thanks @lovebes)\n\n\n## [1.6.0] - 2021-10-26\n\nThis release brings a more resilient parsing of JWK sets. If the provider has encryption keys along with signing keys, we skip them.\n\nAlso, since JWS specification has some algorithms that are not natively available to all installations (Edwards curves for example) we also skip those. This aims to avoid a server not loading other keys if one is not supported.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Changed - Changelog","doc":"- Update Hackney spec to include 1.18 versions (thanks to @J3RN)\n- Add patch level to 1.18 spec (thanks to @J3RN)\n- More resilient parsing of JWKs (#28)\n\n## [1.5.0] - 2020-12-23","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Use hackney v1.17.4\n- Documentation re-organization (#22 thanks to @kianmeng)\n- Conditional telemetry version (#23 thanks to @J3RN)\n\n## [1.4.0] - 2020-09-27","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- (Emil Bostijancic) upgrades hackney dependency to 1.16.0 (#17)\n- Updated deps","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- (@seancribbs) Address unmatched return warnings from Dialyzer (#16)\n\n## [1.3.1] - 2020-03-02","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- (@duzzifelipe) #13 fix: change fetch signers spec\n\n## [1.3.0] - 2020-03-02","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- (@duzzifelipe) #12 feat: telemetry middleware for tesla\n\n## [1.2.0] - 2019-11-08","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- (René Mygind Andersen) #7 Adds support for parsing JWKS returned with content-type 'applicat…","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- (@ltj) Use hackney v1.15.2\n\n## [1.1.0] - 2019-03-05","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Options for explicitly telling which algorithm will use for the parsed signers;\n- HTTP options for retry and adapter;\n- Integration tests for Google and Microsoft JWKS endpoints.","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed docs about how to use DefaultStrategyTemplate and Fixed spelling (#4 thanks to @bforchhammer)\n\n## [1.0.0] - 2019-01-02","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- First version of the library with a default time window strategy for refetching signers.","ref":"changelog.html#added"},{"type":"extras","title":"License","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright [yyyy] [name of copyright owner]\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"},{"type":"extras","title":"Overview","doc":"# Joken JWKS\n\n[![CI](https://github.com/joken-elixir/joken/actions/workflows/ci.yml/badge.svg)](https://github.com/joken-elixir/joken_jwks/actions/workflows/ci.yml)\n[![Module Version](https://img.shields.io/hexpm/v/joken_jwks.svg)](https://hex.pm/packages/joken_jwks)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/joken_jwks/)\n[![Total Download](https://img.shields.io/hexpm/dt/joken_jwks.svg)](https://hex.pm/packages/joken_jwks)\n[![License](https://img.shields.io/hexpm/l/joken_jwks.svg)](https://github.com/joken-elixir/joken_jwks/blob/master/LICENSE)\n[![Last Updated](https://img.shields.io/github/last-commit/joken-elixir/joken_jwks.svg)](https://github.com/joken-elixir/joken_jwks/commits/master)\n\nA `Joken.Hooks` implementation that builds a signer out of a JWKS url for verification.","ref":"readme.html"},{"type":"extras","title":"Usage - Overview","doc":"Please see our [documentation](https://hexdocs.pm/joken_jwks/) for usage.","ref":"readme.html#usage"},{"type":"extras","title":"Installation - Overview","doc":"```elixir\ndef deps do\n  [\n    {:joken_jwks, \"~> 1.6.0\"}\n  ]\nend\n```\n\n# JWKS (JSON Web Key Set)\n\nWhen using JWTs for digital signing we need a key to sign and verify the contents of the token. Many times the part that is doing signature verification is the same that does the signing and so, it possesses both the private and public key. Other times, this is not the case.\n\nWhen using an authentication server that is different than the \"business\" server, the later has no control about the keys being used to authenticate a request. On these cases, the authentication server publishes its own public key so that anyone can validate tokens that it has generated before with the matching private key.\n\nThis scenario is common on delegated authentication as mentioned. One specification that uses this approach is OAuth2 with OpenID Connect. The auth server has a public configuration endpoint where there is plenty information about how to connect with the server. One such information is the JWKS (JSON Web Key Set): a list of public keys the server uses to generate signed tokens.\n\nExample of well known JWKS URIs:\n\n- Google: https://www.googleapis.com/oauth2/v3/certs\n- Microsoft: https://login.microsoftonline.com/common/discovery/keys","ref":"readme.html#installation"},{"type":"extras","title":"Joken JWKS hook - Overview","doc":"If your server is using Joken 2 to validate tokens, then you can use this hook to retrieve the list of signers from a JWKS URI. This is how it happens:\n\n1. Build you token configuration (either with `use Joken.Config` or directly)\n2. Initialize your fetching strategy\n3. Pass this hook to your verify call:\n   - If you are using `Joken.Config` then it's just a matter of `add_hook(JokenJwks, strategy: < >)`\n   - If you are calling `Joken.verify_*` directly, you can pass the hook as a last parameter\n\n**Remember** that there can't be a default signer otherwise it will have precedence over this!","ref":"readme.html#joken-jwks-hook"},{"type":"extras","title":"Architecture - Overview","doc":"This hook will call the behaviour `JokenJwks.SignerMatchstrategy` for every token. It is the implementation job to decide how to choose a signer for the token.\n\nThe only occasion where this will not call the behaviour is if the given token is not properly formed nor it contains a \"kid\" claim on its header. JWKS mandates this claim so that we know which key to use for verifying.\n\nA very naive approach of implementing the callback would be to fetch signers upon startup and then re-fetching every time a token kid does not match with the loaded cache.\n\nThis could potentially open an attack vector for massively hitting the authentication server. Of course, the auth server JWKS url is public and an attacker could just hit it directly, but it is wise to have some defense mechanism in place when developing your strategy.","ref":"readme.html#architecture"},{"type":"extras","title":"Default Strategy Template - Overview","doc":"`JokenJwks` comes with a smart enough implementation that uses a time window approach for re-fetching signers. By default, it polls the cache state every minute to see if a bad kid was attempted. If so, it re-fetches the cache. So, it will fetch JWKS once every minute tops.","ref":"readme.html#default-strategy-template"},{"type":"extras","title":"Interpretation of the JWKS RFC - Overview","doc":"Since the JWKS specification is just that, a specification, many servers might disagree on how to implement this. For example, Google specifies the \"alg\" claim on every key instance. Microsoft does not. Therefore we assume some interpretations:\n\n- Every key must have a \"kid\" (even if there is only one key)\n- We skip keys that are used for encryption (which have a field \"use\" with value \"enc\")\n- We skip keys that have JWE algs or unsupported JWS algorithms\n- If no \"alg\" claim is provided, then the user must pass the option \"explicit_alg\"\n\nThat's it for now :)","ref":"readme.html#interpretation-of-the-jwks-rfc"},{"type":"extras","title":"Copyright and License - Overview","doc":"Copyright (c) 2018 Victor Nascimento\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at [http://www.apache.org/licenses/LICENSE-2.0](http://www.apache.org/licenses/LICENSE-2.0)\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"readme.html#copyright-and-license"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}